@@ -176,16 +176,11 @@
 #define ALLOC_FRAME(n)                                              \
   do {                                                              \
     sp = VP->fp - (n);                                              \
-    if (sp < VP->sp_min_since_gc)                                   \
+    if (SCM_UNLIKELY (sp < VP->stack_limit))                        \
       {                                                             \
-        if (SCM_UNLIKELY (sp < VP->stack_limit))                    \
-          {                                                         \
-            SYNC_IP ();                                             \
-            CALL_INTRINSIC (expand_stack, (thread, sp));            \
-            CACHE_SP ();                                            \
-          }                                                         \
-        else                                                        \
-          VP->sp_min_since_gc = VP->sp = sp;                        \
+        SYNC_IP ();                                                 \
+        CALL_INTRINSIC (expand_stack, (thread, sp));                \
+        CACHE_SP ();                                                \
       }                                                             \
     else                                                            \
       VP->sp = sp;                                                  \
@@ -195,12 +190,7 @@
    stack expansion is needed.  Note that in some cases this may lower
    SP, e.g. after a return but where there are more locals below, but we
    know it was preceded by an alloc-frame in that case, so no stack need
-   be allocated.
-
-   As an optimization, we don't update sp_min_since_gc in this case; the
-   principal place stacks are expanded is in ALLOC_FRAME. it doesn't
-   need to strictly be the min since GC, as it's just an optimization to
-   prevent passing too-large of a range to madvise.  */
+   be allocated.  */
 #define RESET_FRAME(n)                                              \
   do {                                                              \
     VP->sp = sp = VP->fp - (n);                                     \
@@ -3280,9 +3270,55 @@ VM_NAME (scm_thread *thread)
       NEXT (2);
     }
 
-  VM_DEFINE_OP (157, unused_157, NULL, NOP)
-  VM_DEFINE_OP (158, unused_158, NULL, NOP)
-  VM_DEFINE_OP (159, unused_159, NULL, NOP)
+  /* allocate-pointerless-words dst:12 count:12
+   *
+   * Allocate a fresh object consisting of COUNT words and store it into
+   * DST.  The result will not be traced by GC.  COUNT is a u64 local.
+   */
+  VM_DEFINE_OP (157, allocate_pointerless_words, "allocate-pointerless-words", DOP1 (X8_S12_S12))
+    {
+      uint16_t dst, size;
+
+      UNPACK_12_12 (op, dst, size);
+
+      SYNC_IP ();
+      SP_SET (dst, CALL_INTRINSIC (allocate_pointerless_words,
+                                   (thread, SP_REF_U64 (size))));
+      NEXT (1);
+    }
+
+  /* allocate-words/immediate dst:12 count:12
+   *
+   * Allocate a fresh object consisting of COUNT words and store it into
+   * DST.  The result will not be traced by GC.  COUNT is an immediate.
+   */
+  VM_DEFINE_OP (158, allocate_pointerless_words_immediate, "allocate-pointerless-words/immediate", DOP1 (X8_S12_C12))
+    {
+      uint16_t dst, size;
+
+      UNPACK_12_12 (op, dst, size);
+
+      SYNC_IP ();
+      SP_SET (dst, CALL_INTRINSIC (allocate_pointerless_words, (thread, size)));
+
+      NEXT (1);
+    }
+
+  /* s64->f64 dst:12 src:12
+   *
+   * Convert an s64 value to a double-precision floating-point value.
+   */
+  VM_DEFINE_OP (159, s64_to_f64, "s64->f64", DOP1 (X8_S12_S12))
+    {
+      uint16_t dst, src;
+
+      UNPACK_12_12 (op, dst, src);
+
+      SP_SET_F64 (dst, (double) SP_REF_S64 (src));
+
+      NEXT (1);
+    }
+
   VM_DEFINE_OP (160, unused_160, NULL, NOP)
   VM_DEFINE_OP (161, unused_161, NULL, NOP)
   VM_DEFINE_OP (162, unused_162, NULL, NOP)
